@@ -1,12 +1,8 @@
 """
 Generic support for objects with full-featured Parameters and
-messaging.  
-
-$Id$
+messaging.
 """
-__version__='$Revision$'
 
-import sys
 import copy
 import re
 
@@ -14,15 +10,23 @@ from operator import itemgetter,attrgetter
 from types import FunctionType
 from functools import partial, wraps
 
-# JABALERT: Could consider using Python's logging facilities instead.
-SILENT  = 0
-WARNING = 50
-NORMAL  = 100
-MESSAGE = NORMAL
-VERBOSE = 200
-DEBUG   = 300
+import logging
 
-min_print_level = NORMAL
+from logging import DEBUG, INFO, WARNING, ERROR, CRITICAL # pyflakes:ignore (API import)
+VERBOSE = INFO - 1
+logging.addLevelName(VERBOSE, "VERBOSE")
+
+# Logger instance to use for param; if "logger" is set to None, the root logger
+# will be used.
+logger = None
+def get_logger():
+    if logger is None:
+        # If it was not configured before, do default initialization
+        if not logging.getLogger().handlers:
+            logging.basicConfig(level=INFO)
+        return logging.getLogger()
+    else:
+        return logger
 
 # Indicates whether warnings should be raised as errors, stopping
 # processing.
@@ -70,31 +74,37 @@ def get_all_slots(class_):
     # in its superclass (the superclass' __slots__ end up as
     # attributes of the subclass).
     all_slots = []
-    parent_param_classes = [class_ for class_ in classlist(class_)[1::]]
-    for class_ in parent_param_classes:
-        if hasattr(class_,'__slots__'):
-            all_slots+=class_.__slots__
+    parent_param_classes = [c for c in classlist(class_)[1::]]
+    for c in parent_param_classes:
+        if hasattr(c,'__slots__'):
+            all_slots+=c.__slots__
     return all_slots
 
 
 def get_occupied_slots(instance):
     """
     Return a list of slots for which values have been set.
-    
+
     (While a slot might be defined, if a value for that slot hasn't
     been set, then it's an AttributeError to request the slot's
     value.)
     """
     return [slot for slot in get_all_slots(type(instance))
             if hasattr(instance,slot)]
-    
+
 
 def all_equal(arg1,arg2):
     """
-    Return a single boolean for arg1==arg2, even for numpy arrays.
+    Return a single boolean for arg1==arg2, even for numpy arrays
+    using element-wise comparison.
 
     Uses all(arg1==arg2) for sequences, and arg1==arg2 otherwise.
+
+    If both objects have an '_infinitely_iterable' attribute, they are
+    not be zipped together and are compared directly instead.
     """
+    if all(hasattr(el, '_infinitely_iterable') for el in [arg1,arg2]):
+        return arg1==arg2
     try:
         return all(a1 == a2 for a1, a2 in zip(arg1, arg2))
     except TypeError:
@@ -104,7 +114,7 @@ def all_equal(arg1,arg2):
 class bothmethod(object): # pylint: disable-msg=R0903
     """
     'optional @classmethod'
-    
+
     A decorator that allows a method to receive either the class
     object (if called on the class) or the instance object
     (if called on the instance) as its first argument.
@@ -113,7 +123,7 @@ class bothmethod(object): # pylint: disable-msg=R0903
     http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/523033.
     """
     # pylint: disable-msg=R0903
-    
+
     def __init__(self, func):
         self.func = func
 
@@ -129,7 +139,7 @@ class ParameterMetaclass(type):
     """
     Metaclass allowing control over creation of Parameter classes.
     """
-    def __new__(mcs,classname,bases,classdict):        
+    def __new__(mcs,classname,bases,classdict):
         # store the class's docstring in __classdoc
         if '__doc__' in classdict:
             classdict['__classdoc']=classdict['__doc__']
@@ -143,7 +153,7 @@ class ParameterMetaclass(type):
         # a __dict__ unless it also defines __slots__.
         if '__slots__' not in classdict:
             classdict['__slots__']=[]
-        
+
         return type.__new__(mcs,classname,bases,classdict)
 
     def __getattribute__(mcs,name):
@@ -153,7 +163,7 @@ class ParameterMetaclass(type):
             return type.__getattribute__(mcs,'__classdoc')
         else:
             return type.__getattribute__(mcs,name)
-        
+
 
 
 # CEBALERT: we break some aspects of slot handling for Parameter and
@@ -190,7 +200,7 @@ class Parameter(object):
     class Bar(Parameterized):
         delta = Parameter(default=0.6, doc='The difference between steps.')
         ...
-        
+
     class Foo(Bar):
         alpha = Parameter(default=0.1, doc='The starting value.')
         sigma = Parameter(default=0.5, doc='The standard deviation.',
@@ -226,7 +236,7 @@ class Parameter(object):
        superclasses; the other values will be inherited.  E.g. if Foo
        declares
 
-        delta = Parameter(default=0.2) 
+        delta = Parameter(default=0.2)
 
        the default value of 0.2 will override the 0.6 inherited from
        Bar, but the doc will be inherited from Bar.
@@ -249,7 +259,7 @@ class Parameter(object):
     the behavior described here.
     """
     __metaclass__ = ParameterMetaclass
-    
+
     # Because they implement __get__ and __set__, Parameters are known
     # as 'descriptors' in Python; see "Implementing Descriptors" and
     # "Invoking Descriptors" in the 'Customizing attribute access'
@@ -284,7 +294,7 @@ class Parameter(object):
     #
     #
     # Be careful when referring to the 'name' of a Parameter:
-    #                                                   
+    #
     # * A Parameterized class has a name for the attribute which is
     #   being represented by the Parameter ('p' in the example above);
     #   in the code, this is called the 'attrib_name'.
@@ -294,7 +304,7 @@ class Parameter(object):
     #   attrib_name for the Parameter); in the code, this is called
     #   the internal_name.
 
-                                                   
+
     # So that the extra features of Parameters do not require a lot of
     # overhead, Parameters are implemented using __slots__ (see
     # http://www.python.org/doc/2.4/ref/slots.html).  Instead of having
@@ -312,10 +322,10 @@ class Parameter(object):
     # Parameterized class owns it. If a Parameter subclass needs
     # to know the owning class, it can declare an 'objtype' slot
     # (which will be filled in by ParameterizedMetaclass)
-                                                   
+
     def __init__(self,default=None,doc=None,precedence=None,  # pylint: disable-msg=R0913
                  instantiate=False,constant=False,readonly=False,
-                 pickle_default_value=True): 
+                 pickle_default_value=True):
         """
         Initialize a new Parameter object: store the supplied attributes.
 
@@ -334,7 +344,7 @@ class Parameter(object):
         In rare cases where the default value should not be pickled,
         set pickle_default_value=False (e.g. for file search paths).
         """
-        self._attrib_name = None  
+        self._attrib_name = None
         self._internal_name = None
         self.precedence = precedence
         self.default = default
@@ -344,7 +354,7 @@ class Parameter(object):
         self._set_instantiate(instantiate)
         self.pickle_default_value = pickle_default_value
 
-        
+
     def _set_instantiate(self,instantiate):
         """Constant parameters must be instantiated."""
         # CB: instantiate doesn't actually matter for read-only
@@ -370,13 +380,13 @@ class Parameter(object):
         """
         # NB: obj can be None (when __get__ called for a
         # Parameterized class); objtype is never None
-        
-        if not obj:
+
+        if obj is None:
             result = self.default
         else:
             result = obj.__dict__.get(self._internal_name,self.default)
         return result
-        
+
 
     def __set__(self,obj,val):
         """
@@ -387,9 +397,9 @@ class Parameter(object):
 
         If called for a Parameterized instance, set the value of
         this Parameter on that instance (i.e. in the instance's
-        __dict__, under the parameter's internal_name). 
+        __dict__, under the parameter's internal_name).
+
 
-        
         If the Parameter's constant attribute is True, only allows
         the value to be set for a Parameterized class or on
         uninitialized Parameterized instances.
@@ -398,7 +408,7 @@ class Parameter(object):
         value to be specified in the Parameter declaration inside the
         Parameterized source code. A read-only parameter also
         cannot be set on a Parameterized class.
-        
+
         Note that until we support some form of read-only
         object, it is still possible to change the attributes of the
         object stored in a constant or read-only Parameter (e.g. the
@@ -409,7 +419,7 @@ class Parameter(object):
         if self.constant or self.readonly:
             if self.readonly:
                 raise TypeError("Read-only parameter '%s' cannot be modified"%self._attrib_name)
-            elif not obj:
+            elif obj is None:  #not obj
                 self.default = val
             elif not obj.initialized:
                 obj.__dict__[self._internal_name] = val
@@ -417,20 +427,20 @@ class Parameter(object):
                 raise TypeError("Constant parameter '%s' cannot be modified"%self._attrib_name)
 
         else:
-            if not obj:
+            if obj is None:
                 self.default = val
             else:
                 obj.__dict__[self._internal_name] = val
-                
+
 
     def __delete__(self,obj):
         raise TypeError("Cannot delete '%s': Parameters deletion not allowed."%self._attrib_name)
 
 
-    def _set_names(self,attrib_name):        
+    def _set_names(self,attrib_name):
         self._attrib_name = attrib_name
         self._internal_name = "_%s_param_value"%attrib_name
-    
+
 
     def __getstate__(self):
         """
@@ -456,7 +466,7 @@ class String(Parameter):
         """Initialize a string parameter."""
         Parameter.__init__(self,default=default,**params)
         self.allow_None = (default is None or allow_None)
-        
+
     def __set__(self,obj,val):
         if not isinstance(val,str) and not (self.allow_None and val is None):
             raise ValueError("String '%s' only takes a string value."%self._attrib_name)
@@ -487,7 +497,7 @@ class ParameterizedMetaclass(type):
     Additionally, a class can declare itself abstract by having an
     attribute __abstract set to True. The 'abstract' attribute can be
     used to find out if a class is abstract or not.
-    """    
+    """
     def __init__(mcs,name,bases,dict_):
         """
         Initialize the class object (not an instance of the class, but
@@ -506,10 +516,9 @@ class ParameterizedMetaclass(type):
 
         # All objects (with their names) of type Parameter that are
         # defined in this class
-        parameters = [(name,obj)
-                      for (name,obj) in dict_.items()
-                      if isinstance(obj,Parameter)]
-        
+        parameters = [(n,o) for (n,o) in dict_.items()
+                      if isinstance(o,Parameter)]
+
         for param_name,param in parameters:
             mcs._initialize_parameter(param_name,param)
 
@@ -517,15 +526,15 @@ class ParameterizedMetaclass(type):
     def _initialize_parameter(mcs,param_name,param):
         # parameter has no way to find out the name a
         # Parameterized class has for it
-        param._set_names(param_name) 
+        param._set_names(param_name)
         mcs.__param_inheritance(param_name,param)
-    
+
 
     # CBENHANCEMENT: Python 2.6 has abstract base classes.
     # http://docs.python.org/whatsnew/2.6.html
     def __is_abstract(mcs):
         """
-        Return True if the class has an attribute __abstract set to True.  
+        Return True if the class has an attribute __abstract set to True.
         Subclasses will return False unless they themselves have
         __abstract set to true.  This mechanism allows a class to
         declare itself to be abstract (e.g. to avoid it being offered
@@ -542,7 +551,7 @@ class ParameterizedMetaclass(type):
             return getattr(mcs,'_%s__abstract'%mcs.__name__)
         except AttributeError:
             return False
-        
+
     abstract = property(__is_abstract)
 
 
@@ -555,7 +564,7 @@ class ParameterizedMetaclass(type):
         that descriptor is a Parameter, and the new value is *not* a
         Parameter, then call that Parameter's __set__ method with the
         specified value.
-        
+
         In all other cases set the attribute normally (i.e. overwrite
         the descriptor).  If the new value is a Parameter, once it has
         been set we make sure that the value is inherited from
@@ -570,9 +579,9 @@ class ParameterizedMetaclass(type):
                 type.__setattr__(mcs,attribute_name,copy.copy(parameter))
             mcs.__dict__[attribute_name].__set__(None,value)
 
-        else:    
+        else:
             type.__setattr__(mcs,attribute_name,value)
-            
+
             if isinstance(value,Parameter):
                 mcs.__param_inheritance(attribute_name,value)
             else:
@@ -591,8 +600,8 @@ class ParameterizedMetaclass(type):
                 if not attribute_name.startswith('_'):
                     print ("Warning: Setting non-Parameter class attribute %s.%s = %s "
                            % (mcs.__name__,attribute_name,`value`))
-                
-                
+
+
     def __param_inheritance(mcs,param_name,param):
         """
         Look for Parameter values in superclasses of this
@@ -632,8 +641,8 @@ class ParameterizedMetaclass(type):
         # class. Such classes can declare an 'objtype' slot, and the
         # owning class will be stored in it.
         if 'objtype' in slots:
-            setattr(param,'objtype',mcs)            
-            del slots['objtype'] 
+            setattr(param,'objtype',mcs)
+            del slots['objtype']
 
         # instantiate is handled specially
         for superclass in classlist(mcs)[::-1]:
@@ -662,7 +671,7 @@ class ParameterizedMetaclass(type):
                     new_value = getattr(new_param,slot)
                     setattr(param,slot,new_value)
 
-        
+
     def get_param_descriptor(mcs,param_name):
         """
         Goes up the class hierarchy (starting from the current class)
@@ -712,10 +721,10 @@ def script_repr(val,imports,prefix,settings):
 
     elif hasattr(val,'script_repr'):
         rep=val.script_repr(imports=imports,prefix=prefix+"    ")
-        
+
     else:
         rep=repr(val)
-        
+
     return rep
 
 
@@ -739,7 +748,7 @@ def container_script_repr(container,imports,prefix,settings):
     rep=d1+','.join(result)+d2
 
     # no imports to add for built-in types
-    
+
     return rep
 
 # why I have to type prefix and settings?
@@ -789,60 +798,42 @@ class Parameterized(object):
     """
     Base class for named objects that support Parameters and message
     formatting.
-    
+
     Automatic object naming: Every Parameterized instance has a name
     parameter.  If the user doesn't designate a name=<str> argument
     when constructing the object, the object will be given a name
     consisting of its class name followed by a unique 5-digit number.
-    
+
     Automatic parameter setting: The Parameterized __init__ method
     will automatically read the list of keyword parameters.  If any
     keyword matches the name of a Parameter (see Parameter class)
     defined in the object's class or any of its superclasses, that
     parameter in the instance will get the value given as a keyword
     argument.  For example:
-    
+
       class Foo(Parameterized):
          xx = Parameter(default=1)
-    
+
       foo = Foo(xx=20)
-    
+
     in this case foo.xx gets the value 20.
-    
+
     Message formatting: Each Parameterized instance has several methods
-    for optionally printing output according to the current 'print
-    level', such as SILENT, WARNING, MESSAGE, VERBOSE, or DEBUG.  Each
-    successive level allows more messages to be printed.  For example,
-    when the level is VERBOSE, all warning, message, and verbose
-    output will be printed.  When it is WARNING, only warnings will be
-    printed.  When it is SILENT, no output will be printed.
-    
-    For each level (except SILENT) there's an associated print method:
-    Parameterized.warning(), .message(), .verbose(), and .debug().
-    
-    Each line printed this way is prepended with the name of the
-    object that printed it.  The Parameterized.print_level parameter
-    and the module global variable min_print_level combine to
-    determine what gets printed.  For example, if foo is a
-    Parameterized:
-    
-       foo.message('The answer is',42)
-    
-    is equivalent to:
-    
-       if max(foo.print_level,parameterized.min_print_level) >= MESSAGE:
-           print foo.name+':', 'The answer is', 42
+    methods for optionally printing output. This functionality is
+    based on the standard Python 'logging' module; using the methods
+    provided here, wraps calls to the 'logging' module's root logger
+    and prepends each message with information about the instance
+    from which the call was made. For more information on how to set
+    the global logging level and change the default message prefix,
+    see documentation for the 'logging' module.
     """
 
     __metaclass__ = ParameterizedMetaclass
 
     name           = String(default=None,constant=True,doc="""
     String identifier for this object.""")
-    
-    ### JABALERT: Should probably make this an Enumeration instead.
-    print_level = Parameter(default=MESSAGE,precedence=-1)
 
-    
+
     def __init__(self,**params):
         """
         Initialize this Parameterized instance.
@@ -862,7 +853,7 @@ class Parameterized(object):
         self.initialized=False
 
         self.__generate_name()
-        
+
         self._setup_params(**params)
         object_count += 1
 
@@ -891,7 +882,7 @@ class Parameterized(object):
         cls.__metaclass__._initialize_parameter(cls,param_name,param_obj)
         # delete cached params()
         try:
-            delattr(cls,'_%s__params'%cls.__name__) 
+            delattr(cls,'_%s__params'%cls.__name__)
         except AttributeError:
             pass
 
@@ -909,14 +900,14 @@ class Parameterized(object):
             raise ValueError("'%s' is not a parameter of %s"%(param_name,self_or_cls))
         setattr(self_or_cls,param_name,val)
 
-        
+
     # CEBALERT: I think I've noted elsewhere the fact that we
     # sometimes have a method on Parameter that requires passing the
     # owning Parameterized instance or class, and other times we have
     # the method on Parameterized itself.  In case I haven't written
     # that down elsewhere, here it is again.  We should clean that up
     # (at least we should be consistent).
-    
+
     # cebalert: it's really time to stop and clean up this bothmethod
     # stuff and repeated code in methods using it.
 
@@ -963,7 +954,7 @@ class Parameterized(object):
                 g._Dynamic_time = g._saved_Dynamic_time.pop()
             elif hasattr(g,'state_pop') and isinstance(g,Parameterized):
                 g.state_pop()
-        
+
 
     @classmethod
     def set_default(cls,param_name,value):
@@ -973,7 +964,7 @@ class Parameterized(object):
         Equivalent to setting param_name on the class.
         """
         setattr(cls,param_name,value)
-    
+
 
     @bothmethod
     def set_dynamic_time_fn(self_or_cls,time_fn,sublistattr=None):
@@ -1012,11 +1003,11 @@ class Parameterized(object):
                 sublist = getattr(self_or_cls,sublistattr)
             except AttributeError:
                 sublist = []
-        
+
             for obj in sublist:
                 obj.set_dynamic_time_fn(time_fn,sublistattr)
-                
-            
+
+
     @as_uninitialized
     def _set_name(self,name):
         self.name=name
@@ -1035,7 +1026,7 @@ class Parameterized(object):
         """
         Provide a nearly valid Python representation that could be used to recreate
         the item with its parameters, if executed in the appropriate environment.
-        
+
         Returns 'classname(parameter1=x,parameter2=y,...)', listing
         all the parameters of this object.
         """
@@ -1049,22 +1040,20 @@ class Parameterized(object):
     def script_repr(self,imports=[],prefix="    "):
         """
         Variant of __repr__ designed for generating a runnable script.
-        """        
-        # Suppresses automatically generated names and print_levels.
+        """
+        # Suppresses automatically generated names.
         settings=[]
         for name,val in self.get_param_values(onlychanged=script_repr_suppress_defaults):
             if name == 'name' and (val is not None and
                                    re.match('^'+self.__class__.__name__+'[0-9]+$',val)):
                 rep=None
-            elif name == 'print_level':
-                rep=None
             else:
                 rep=script_repr(val,imports,prefix,settings)
 
             if rep is not None:
                 settings.append('%s=%s' % (name,rep))
 
-            
+
         # Generate import statement
         mod = self.__module__
 
@@ -1078,7 +1067,7 @@ class Parameterized(object):
         # updated this code in other places (e.g. simulation).
         return mod+'.'+self.__class__.__name__ + "(" + (",\n"+prefix).join(settings) + ")"
 
-        
+
     def __str__(self):
         """Return a short representation of the name and class of this object."""
         return "<%s %s>" % (self.__class__.__name__,self.name)
@@ -1095,36 +1084,29 @@ class Parameterized(object):
     # Note that Python's logging module would simplify print
     # statements still further (see "topographica's debug printing"
     # emails between CB&JB).
-    def __db_print(self,level=NORMAL,*args):
+    def __db_print(self,level=INFO,*args):
         """
-        Print each of the given args iff print_level or
-        self.db_print_level is greater than or equal to the given
-        level.
-
         Any of args may be functions, in which case they will be
         called. This allows delayed execution, preventing
         time-consuming code from being called unless the print level
         requires it. (The time-consuming code is usually that used to
         build the repr().)
         """
-        if level <= max(min_print_level,self.print_level):
+        if get_logger().isEnabledFor(level):
 
             # call any args that are functions
             args = list(args)
             for a in args:
                 if isinstance(a,FunctionType): args[args.index(a)]=a()
-            
+
             s = ' '.join([str(x) for x in args])
-            
+
             if dbprint_prefix and callable(dbprint_prefix):
                 prefix=dbprint_prefix() # pylint: disable-msg=E1102
             else:
                 prefix=""
-                
-            print "%s%s: %s" % (prefix,self.name,s)
-            
-        sys.stdout.flush()
 
+            get_logger().log(level, "%s%s: %s" % (prefix,self.name,s))
 
     def warning(self,*args):
         """
@@ -1135,15 +1117,15 @@ class Parameterized(object):
         if not warnings_as_exceptions:
             global warning_count
             warning_count+=1
-            self.__db_print(WARNING,"Warning:",*args)
+            self.__db_print(WARNING,*args)
         else:
             raise Exception, ' '.join(["Warning:",]+[str(x) for x in args])
 
 
     def message(self,*args):
         """Print the arguments as a message."""
-        self.__db_print(MESSAGE,*args)
-        
+        self.__db_print(INFO,*args)
+
     def verbose(self,*args):
         """Print the arguments as a verbose message."""
         self.__db_print(VERBOSE,*args)
@@ -1167,7 +1149,7 @@ class Parameterized(object):
             # CB: writes over name given to the original object;
             # should it instead keep the same name?
             new_object.__generate_name()
-        
+
 
     @as_uninitialized
     def _setup_params(self,**params):
@@ -1182,7 +1164,7 @@ class Parameterized(object):
         parameters.
 
         Constant Parameters can be set during calls to this method.
-        """        
+        """
         ## Deepcopy all 'instantiate=True' parameters
         # (build a set of names first to avoid redundantly instantiating
         #  a later-overridden parent class's parameter)
@@ -1255,12 +1237,12 @@ class Parameterized(object):
             cls = cls_or_slf
         else:
             slf = cls_or_slf
-            
-        if not hasattr(param_obj,'_force'): 
+
+        if not hasattr(param_obj,'_force'):
             return param_obj.__get__(slf,cls)
         else:
-            return param_obj._force(slf,cls) 
-            
+            return param_obj._force(slf,cls)
+
 
     @bothmethod
     def get_value_generator(cls_or_slf,name): # pylint: disable-msg=E0213
@@ -1281,7 +1263,7 @@ class Parameterized(object):
         elif hasattr(param_obj,'attribs'):
             value = [cls_or_slf.get_value_generator(a) for a in param_obj.attribs]
 
-        # not a Dynamic Parameter 
+        # not a Dynamic Parameter
         elif not hasattr(param_obj,'_value_is_dynamic'):
             value = getattr(cls_or_slf,name)
 
@@ -1321,7 +1303,7 @@ class Parameterized(object):
                 value = param_obj._inspect(cls_or_slf,None)
 
         return value
-            
+
 
 
     def print_param_values(self):
@@ -1336,7 +1318,7 @@ class Parameterized(object):
         copy of the object's __dict__ and that also includes the
         object's __slots__ (if it has any).
         """
-        # remind me, why is it a copy? why not just state.update(self.__dict__)?        
+        # remind me, why is it a copy? why not just state.update(self.__dict__)?
         state = self.__dict__.copy()
 
         for slot in get_occupied_slots(self):
@@ -1360,7 +1342,7 @@ class Parameterized(object):
         """
         self.initialized=False
         for name,value in state.items():
-            setattr(self,name,value)                
+            setattr(self,name,value)
         self.initialized=True
 
 
@@ -1390,12 +1372,12 @@ class Parameterized(object):
             # _Parameterized.__params for all classes).
             setattr(cls,'_%s__params'%cls.__name__,paramdict)
             pdict= paramdict
-        
+
         if parameter_name is None:
             return pdict
         else:
             return pdict[parameter_name]
-       
+
 
 
     @classmethod
@@ -1433,20 +1415,20 @@ class Parameterized(object):
 ##         # Deepcopy all attributes in __slots__ and __dict__, except
 ##         # for attributes which are ObjectSelector parameters (which
 ##         # are not copied at all).
-##         # 
+##         #
 ##         # Should be equivalent to copy.deepcopy(self), but without copying
 ##         # ObjectSelector parameters.
-        
+
 ##         if memo is None:
 ##             memo = {}
 
 ##         class_ = self.__class__
 ##         new_instance = class_.__new__(class_)
-        
+
 ##         memo[id(self)]=new_instance
 
 ##         ## attributes are in __dict__ and __slots__
-##         all_attributes = []        
+##         all_attributes = []
 ##         if hasattr(self,'__dict__'):
 ##             all_attributes+=self.__dict__.keys()
 ##         if hasattr(self,'__slots__'):
@@ -1468,10 +1450,10 @@ class Parameterized(object):
 ##             setattr(new_instance,attr,obj)
 
 ##         return new_instance
-        
-        
 
-        
+
+
+
 def print_all_param_defaults():
     """Print the default values for all imported Parameters."""
     print "_______________________________________________________________________________"
@@ -1502,10 +1484,10 @@ class ParamOverrides(dict):
     # NOTE: Attribute names of this object block parameters of the
     # same name, so all attributes of this object should have names
     # starting with an underscore (_).
-    
+
     def __init__(self,overridden,dict_,allow_extra_keywords=False):
         """
-        
+
         If allow_extra_keywords is False, then all keys in the
         supplied dict_ must match parameter names on the overridden
         object (otherwise a warning will be printed).
@@ -1536,11 +1518,11 @@ class ParamOverrides(dict):
         overridden object.
         """
         return self._extra_keywords
-    
+
     def __missing__(self,name):
         # Return 'name' from the overridden object
         return getattr(self._overridden,name)
-        
+
     def __repr__(self):
         # As dict.__repr__, but indicate the overridden object
         return dict.__repr__(self)+" overriding params from %s"%repr(self._overridden)
@@ -1559,7 +1541,7 @@ class ParamOverrides(dict):
             self.__setitem__(name,val)
         else:
             dict.__setattr__(self,name,val)
-            
+
     def _check_params(self,params):
         """
         Print a warning if params contains something that is not a
@@ -1586,8 +1568,10 @@ class ParamOverrides(dict):
         return extra_keywords
 
 
-def _new_parameterized(*a,**k):
-    return Parameterized.__new__(*a,**k)
+# Helper function required by ParameterizedFunction.__reduce__
+def _new_parameterized(cls):
+    return Parameterized.__new__(cls)
+
 
 class ParameterizedFunction(Parameterized):
     """
@@ -1606,10 +1590,23 @@ class ParameterizedFunction(Parameterized):
     def __str__(self):
         return self.__class__.__name__+"()"
 
-    @classmethod
-    def instance(class_,*args,**params):
-        """Return an instance of this class."""
-        inst=Parameterized.__new__(class_,*args)
+    @bothmethod
+    def instance(self_or_cls,**params):
+        """
+        Return an instance of this class, copying parameters from any
+        existing instance provided.
+        """
+
+        if isinstance (self_or_cls,ParameterizedMetaclass):
+            cls = self_or_cls
+        else:
+            p = params
+            params = dict(self_or_cls.get_param_values())
+            params.update(p)
+            params.pop('name')
+            cls = self_or_cls.__class__
+
+        inst=Parameterized.__new__(cls)
         Parameterized.__init__(inst,**params)
         return inst
 
@@ -1639,8 +1636,8 @@ class ParameterizedFunction(Parameterized):
         """
         r = Parameterized.script_repr(self,imports,prefix)
         classname=self.__class__.__name__
-        return r.replace(".%s("%classname,".%s.instance("%classname)    
-        
+        return r.replace(".%s("%classname,".%s.instance("%classname)
+
 
 
 
@@ -1668,7 +1665,7 @@ class overridable_property(object):
         self.fset = fset
         self.fdel = fdel
         self.__doc__ = doc
- 
+
     def __get__(self, obj, objtype=None):
         if obj is None:
             return self
@@ -1678,7 +1675,7 @@ class overridable_property(object):
             return self.fget(obj)
         else:
             return getattr(obj, self.fget.__name__)()
- 
+
     def __set__(self, obj, value):
         if self.fset is None:
             raise AttributeError, "can't set attribute"
@@ -1686,7 +1683,7 @@ class overridable_property(object):
             self.fset(obj, value)
         else:
             getattr(obj, self.fset.__name__)(value)
- 
+
     def __delete__(self, obj):
         if self.fdel is None:
             raise AttributeError, "can't delete attribute"
